#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>
#include <sys/time.h>
#define NET_LENGTH 85
#define PACKET_LENGTH 257
#define MAX_CLIENTS 20

#ifndef KBTELNET
struct connected_client_vars
{ 
  int number_of_clients;
  char client_fd[MAX_CLIENTS][5];
  char client_name[MAX_CLIENTS][31];
  char client_alias[MAX_CLIENTS][21];
  char client_addr_dq[MAX_CLIENTS][16];
 };
static struct connected_client_vars ccv = {.number_of_clients = 0};
#endif
