static void get_country(gint qnum, gchar *call_px);
static gint get_packet_country(const gchar *packet_call, gchar *call_px);
static void get_your_country(void);
static gint get_locus_index(gint iqnum);
static gint get_locus_index_cr(void);
static gint get_worked_locus(void);
static gint get_country_index(gint iqnum);
static gint get_packet_country_index(const gchar *pbm_country);
static gint get_zone_index(gint iqnum);
static gint get_zi_from_rn(gint qnum);
static void get_worked_country(gint qnum, gchar *call_px);
static gint get_continent_index(gint iqnum);
static void pick_country_box(GtkWidget *widget, gint idraw);
static gint pcb_key_handler(GtkWidget *widget, GdkEventKey *event,
                            gpointer da);
static void pick_zone_box(GtkWidget *widget, gint idraw);
static gint pzb_key_handler(GtkWidget *widget, GdkEventKey *event,
                            gpointer da);
static void check_for_conflict(GtkWidget *widget);
